﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;

namespace Prova2
{
    class FormGenerator
    {
        private readonly Form _form;
        private readonly ILogger _logger;
        private Type _controlType;
        private Control _control;

        public FormGenerator(Form form)
            : this(form, null)
        {
        }

        public FormGenerator(Form form, ILogger logger)
        {
            if (form == null)
                throw new ArgumentNullException("form");
            _form = form;
            _logger = logger;
        }

        public void Load(string fileName)
        {
            XmlDocument document = new XmlDocument();
            document.Load(fileName);
            foreach (XmlElement controlElement in document.SelectNodes("/Form/Control"))
            {
                string controlType = controlElement.GetAttribute("type");
                string controlAssembly = controlElement.GetAttribute("assembly");
                CreateControl(controlType, controlAssembly);
                if (_control == null)
                    continue;
                foreach (XmlElement propertyElement in controlElement.SelectNodes("*"))
                {
                    string propertyName = propertyElement.Name;
                    string propertyValue = propertyElement.InnerText;
                    SetControlProperty(propertyName, propertyValue);
                }
                AddControlToForm();
            }
        }

        public void Load1(string fileName)
        {
            using (XmlReader reader = new XmlTextReader(fileName))
            {
                while (reader.Read())
                {
                    if (reader.IsStartElement("Control"))
                    {
                        string controlType = reader.GetAttribute("type");
                        string controlAssembly = reader.GetAttribute("assembly");
                        CreateControl(controlType, controlAssembly);
                        if (_control == null)
                        {
                            reader.Skip();
                            continue;
                        }
                        reader.ReadStartElement();
                        while (reader.IsStartElement())
                        {
                            string propertyName = reader.LocalName;
                            string propertyValue = reader.ReadElementString();
                            SetControlProperty(propertyName, propertyValue);
                        }
                        AddControlToForm();
                    }
                }
            }
        }

        //
        //Creazione del controllo:
        //  Tipo = System.Windows.Forms.Button
        //  Assembly = System.Windows.Forms, version=1.0.3300.0, Culture=neutral, PublicKeyToken=b77a5c561934e089
        //-- Creazione fallita!
        private void CreateControl(string controlType, string controlAssembly)
        {
            WriteLine("");
            WriteLine("Creazione del controllo:");
            WriteLine("  Tipo = " + controlType);
            WriteLine("  Assembly = " + controlAssembly);
            try
            {
                Assembly assembly = Assembly.Load(controlAssembly);
                _controlType = Type.GetType(String.Format("{0},{1}", controlType, controlAssembly), true);
                _control = (Control) Activator.CreateInstance(_controlType);
            }
            catch
            {
                WriteLine("-- Creazione fallita!");
                _controlType = null;
                _control = null;
            }
        }
        //Proprietà: Top, valore: 200
        //-- Proprietà errata o non esistente!
        private void SetControlProperty(string propertyName, string propertyValue)
        {
            WriteLine(String.Format("Proprietà: {0}, valore: {1}", propertyName, propertyValue));
            try
            {
                PropertyInfo propertyInfo = _controlType.GetProperty(propertyName);
                object value = Convert.ChangeType(propertyValue, propertyInfo.PropertyType);
                propertyInfo.SetValue(_control, value, null);
            }
            catch
            {
                WriteLine("-- Proprietà errata o non esistente!");
            }
        }
        //Creazione Ok!
        private void AddControlToForm()
        {
            WriteLine("Creazione Ok!");
            _form.Controls.Add(_control);
        }

        public void LinkHandler()
        {
            foreach (Control control in _form.Controls)
            {
                if (control is Button)
                {
                    switch (control.Text)
                    {
                        case "Start":
                            control.Click += StartHandler;
                            break;
                        case "Stop":
                            control.Click += StopHandler;
                            break;
                    }
                }
            }
        }

        private void StartHandler(object sender, EventArgs e)
        {
            InvokeHandler("Start");
        }

        private void StopHandler(object sender, EventArgs e)
        {
            InvokeHandler("Stop");
        }

        private void InvokeHandler(string methodName)
        {
            foreach (Control control in _form.Controls)
            {
                Type controlType = control.GetType();
                MethodInfo methodInfo = controlType.GetMethod(methodName);
                if (methodInfo != null)
                    methodInfo.Invoke(control, null);
            }
        }

        private void WriteLine(string line)
        {
            if (_logger != null)
                _logger.WriteLine(line);
        }
    }
}
